/*
  Fin - a simple personal accounts manager
  Copyright (C) 2012  Bob Mottram <bob@sluggish.dyndns.org>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "piechart.h"

int piechart_get_income(char * account, char * search_string,
						int month_number, int year,
						char * date_format, char * currency,
						double * total)
{
	int retval, col, cols, i;
	char database_filename[STRING_BLOCK];
	char query[STRING_BLOCK];
	char fields[STRING_BLOCK];
	time_t now;
	struct tm *d;
	sqlite3_stmt *stmt;
	sqlite3 *handle;

	*total=0;

	sprintf((char*)fields,"%s","sum(receive) as TotalReceive");

	/* wildcards */
	for (i=0;i<strlen(search_string);i++) {
		if (search_string[i]=='*') {
			search_string[i]='%';
		}
	}

	if ((month_number==0) && (year==0)) {
		sprintf((char*)query,
				"select %s from transactions where " \
				"(currency is \"%s\");",fields,currency);
	}
	else {
		if (year==0) {
			time(&now);
			d = localtime(&now);
			year = YEAR_BASELINE + d->tm_year;
			if (month_number > d->tm_mon+1) year--;
		}

		if (strcmp(search_string,"*")==0) {
			if (month_number>0) {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"(date like \"%d-%02d-%c\");",
						fields,currency,year,month_number,'%');
			}
			else {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"(date like \"%d-%c-%c\");",
						fields,currency,year,'%','%');
			}
		}
		else {
			if (month_number>0) {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"(date like \"%d-%02d-%c\");",
						fields,currency,year,month_number,'%');
			}
			else {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"(date like \"%d-%c-%c\");",
						fields,currency,year,'%','%');
			}
		}
	}

	/* if no account is specified use the default one */
	if (strlen(account)==0) {
		settings_set_account(get_text_from_identifier(SETTINGS_DEFAULT_CURRENT_ACCOUNT));
		if (strlen(get_text_from_identifier(SETTINGS_DEFAULT_CURRENT_ACCOUNT))==0) {
			printf("Unable to get default account name\n");
			return 0;
		}
		account = settings_get_account();
	}

	if (strlen(account)==0) {
		printf("No account specified\n");
		return 0;
	}
	sprintf((char*)database_filename,
			"%s/.fin/%s.sqlite3",getenv("HOME"),account);
	retval = sqlite3_open_v2(database_filename,
							 &handle,
							 SQLITE_OPEN_READWRITE|SQLITE_OPEN_CREATE,
							 NULL);
	/* If connection failed, handle returns NULL */
	if (retval) {
		printf("Sqlite3 database connection failed\n");
		return 0;
	}

	retval = sqlite3_prepare_v2(handle,query,-1,&stmt,0);
	if (retval) {
		printf("Unable to retrieve search information.  " \
			   "Does this account '%s' exist?\n",account);
		return 0;
	}
    
	cols = sqlite3_column_count(stmt);

	while(1) {
		retval = sqlite3_step(stmt);
        
		if(retval == SQLITE_ROW) {

			for(col=0; col<cols; col++) {
				char *val = (char*)sqlite3_column_text(stmt,col);
				if (col==0) {
					*total += atof(val);
				}
			}
		}
		else if(retval == SQLITE_DONE) {
			break;
		}
	}

	sqlite3_close(handle);

	return 1;
}

int piechart_get_totals(char * account, char * search_string,
						int month_number, int year,
						char * date_format, char * currency,
						double * total_receive, double * total_spend)
{
	int retval, col, cols, i;
	char database_filename[STRING_BLOCK];
	char query[STRING_BLOCK];
	char fields[STRING_BLOCK];
	time_t now;
	struct tm *d;
	sqlite3_stmt *stmt;    
	sqlite3 *handle;

	*total_receive=0;
	*total_spend=0;

	sprintf((char*)fields,"%s",
			"sum(receive) as TotalReceive, sum(spend) as TotalSpend");

	/* wildcards */
	for (i=0;i<strlen(search_string);i++) {
		if (search_string[i]=='*') {
			search_string[i]='%';
		}
	}

	if ((month_number==0) && (year==0)) {
		sprintf((char*)query,
				"select %s from transactions where " \
				"(currency is \"%s\") and ((description like \"%s\")" \
				" or (date like \"%s\"));",
				fields,currency,search_string,search_string);
	}
	else {
		if (year==0) {
			time(&now);
			d = localtime(&now);
			year = YEAR_BASELINE + d->tm_year;
			if (month_number > d->tm_mon+1) year--;
		}

		if (strcmp(search_string,"*")==0) {
			if (month_number>0) {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"(date like \"%d-%02d-%c\");",
						fields,currency,year,month_number,'%');
			}
			else {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"(date like \"%d-%c-%c\");",
						fields,currency,year,'%','%');
			}
		}
		else {
			if (month_number>0) {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"((description like \"%s\") and " \
						"(date like \"%d-%02d-%c\"));",
						fields,currency,search_string,
						year,month_number,'%');
			}
			else {
				sprintf((char*)query,
						"select %s from transactions where " \
						"(currency is \"%s\") and " \
						"((description like \"%s\") and " \
						"(date like \"%d-%c-%c\"));",
						fields,currency,search_string,year,'%','%');
			}
		}
	}

	/* if no account is specified use the default one */
	if (strlen(account)==0) {
		settings_set_account(get_text_from_identifier(SETTINGS_DEFAULT_CURRENT_ACCOUNT));
		if (strlen(get_text_from_identifier(SETTINGS_DEFAULT_CURRENT_ACCOUNT))==0) {
			printf("Unable to get default account name\n");
			return 0;
		}
		account = settings_get_account();
	}

	if (strlen(account)==0) {
		printf("No account specified\n");
		return 0;
	}
	sprintf((char*)database_filename,
			"%s/.fin/%s.sqlite3",getenv("HOME"),account);
	retval = sqlite3_open_v2(database_filename,
							 &handle,
							 SQLITE_OPEN_READWRITE|SQLITE_OPEN_CREATE,
							 NULL);
	/* If connection failed, handle returns NULL */
	if (retval) {
		printf("Sqlite3 database connection failed\n");
		return 0;
	}

	retval = sqlite3_prepare_v2(handle,query,-1,&stmt,0);
	if (retval) {
		printf("Unable to retrieve search information.  " \
			   "Does this account '%s' exist?\n",account);
		return 0;
	}
    
	cols = sqlite3_column_count(stmt);

	while(1) {
		retval = sqlite3_step(stmt);
        
		if(retval == SQLITE_ROW) {

			for(col=0; col<cols; col++) {
				char *val = (char*)sqlite3_column_text(stmt,col);
				if (val!=0) {
					if (col==0) {
						*total_receive += atof(val);
					}
					if (col==1) {
						*total_spend += atof(val);
					}
				}
			}
		}
		else if(retval == SQLITE_DONE) {
			break;
		}
	}

	sqlite3_close(handle);

	return 1;
}

int parse_search_string(char * search_string, char search_separator,
						char field_separator, int start_index,
						char * search, char * fieldname)
{
	int i = start_index;
	int j = 0, k = 0;
	int reading_fieldname = 0;

	while (i < strlen(search_string)) {
		if (search_string[i] != search_separator) {
			if (search_string[i] != field_separator) {
				if (reading_fieldname==0) {
					search[j++] = search_string[i];
				}
				else {
					fieldname[k++] = search_string[i];
				}
			}
			else {
				reading_fieldname = 1;
			}
		}
		else {
			break;
		}
		i++;
	}
	search[j]=0;
	fieldname[k]=0;
	return i+1;
}

int piechart(char * title, char * account, char * search_string,
			 int month_number, int year, char * date_format,
			 char field_separator, char * currency,
			 char * export_filename)
{
	double total_receive=0, total_spend=0;
	double tot_receive=0, tot_spend=0, total_income=0;
	int start_index = 0,i,j;
	char search[STRING_BLOCK];
	char fieldname[STRING_BLOCK];
	double spend[100];
	char field[100][STRING_BLOCK];
	int ctr=0,valid_filename,max_length=10;
	FILE * fp;

	/* get the total income */
	piechart_get_income(account, search_string, month_number,
						year, date_format, currency, &total_income);

	/* perform multiple searches */
	while (start_index < strlen(search_string)) {
		start_index = parse_search_string(search_string, ',', '|',
										  start_index, search,
										  fieldname);
    
		piechart_get_totals(account, search, month_number,
							year, date_format, currency,
							&total_receive, &total_spend);

		tot_receive += total_receive;
		tot_spend += total_spend;
		spend[ctr] = total_spend;
		field[ctr][0]=0;

		if (fieldname[0] != 0) {
			for (i = 0; i < strlen(fieldname); i++) {
				field[ctr][i] = fieldname[i];
			}
			field[ctr][i] = 0;
			if (i>max_length) {
				max_length=i;
			}
		}
		else {
			j = 0;
			for (i = 0; i < strlen(search); i++) {
				if ((search[i] != '*') && (search[i] != '%')) {
					field[ctr][j++] = search[i];
				}
			}
			field[ctr][j]=0;
			if (j>max_length) {
				max_length=j;
			}
		}
		if (field[ctr][0]!=0) {
			field[ctr][0] = toupper(field[ctr][0]);
		}

		ctr++;
	}

	if (ctr<1) {
		return 0;
	}

	if (tot_spend>total_income) {
		total_income = tot_spend;
	}
	if (total_income<=0) {
		printf("%s\n",get_text_from_identifier(WARNING_NO_INCOME));
		return 0;
	}

	valid_filename = 0;
	if (export_filename!=0) {
		if (strlen(export_filename)>0) {
			valid_filename = 1;
		}
	}

	if (valid_filename==0) {
		if (title!=0) {
			printf("%s\n",title);
		}

		sprintf(fieldname,"%s",get_text_from_identifier(CATEGORY));
		pad_text(fieldname,max_length+2);
		fieldname[0]=toupper(fieldname[0]);
		printf("%s",fieldname);
    
		sprintf(fieldname,"%s",get_text_from_identifier(PERCENT));
		fieldname[0]=toupper(fieldname[0]);
		printf("%s\n",fieldname);

		for (i=0;i<max_length+10;i++) {
			printf("-");
		}
		printf("\n");

		for (i=0;i<ctr;i++) {
			j=0;
			while (field[i][j] != 0) {
				fieldname[j] = field[i][j];
				j++;
			}
			fieldname[j] = 0;

			pad_text(fieldname,max_length+2);

			printf("%s",fieldname);
    
			if (spend[i]*100/total_income<10) {
				printf(" ");
			}
			printf("%.1f%c\n",spend[i]*100/total_income,'%');
		}

		sprintf(fieldname,"%s",get_text_from_identifier(CATEGORY_OTHER));
		pad_text(fieldname,max_length+2);
		printf("%s",fieldname);
		if ((total_income-tot_spend)*100/total_income<10) {
			printf(" ");
		}
		printf("%.1f%c\n",(total_income-tot_spend)*100/total_income,'%');

	}
	else {
		fp = fopen(export_filename,"w");
		if (fp!=0) {
			if (title!=0) {
				fprintf(fp,"%s\n",title);
			}

			sprintf(fieldname,"%s",get_text_from_identifier(CATEGORY));
			fieldname[0]=toupper(fieldname[0]);
			fprintf(fp,"%s%c",fieldname,field_separator);
    
			sprintf(fieldname,"%s",get_text_from_identifier(PERCENT));
			fieldname[0]=toupper(fieldname[0]);
			fprintf(fp,"%s\n",fieldname);

			for (i=0;i<ctr;i++) {
				j=0;
				while (field[i][j] != 0) {
					fieldname[j] = field[i][j];
					j++;
				}
				fieldname[j] = 0;

				fprintf(fp,"%s%c",fieldname,field_separator);
    
				fprintf(fp,"%.4f\n",spend[i]*100/total_income);    
			}

			fprintf(fp,"%s%c",
					get_text_from_identifier(CATEGORY_OTHER),
					field_separator);
			fprintf(fp,"%.4f\n",
					(total_income-tot_spend)*100/total_income);

			fclose(fp);
			printf(get_text_from_identifier(EXPORT_COMPLETED),
				   export_filename);
			printf("\n");
		}
	}

	return 1;
}
