/*
  Fin - a simple personal accounts manager
  Copyright (C) 2012  Bob Mottram <bob@sluggish.dyndns.org>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "plot.h"

static int plot_cleanup()
{
	char commandstr[STRING_BLOCK];

	sprintf(commandstr,"%s","rm gnuplot.plot gnuplot*.csv");
	return system(commandstr);
}

int plot(char * title, char * subtitle, int balance_field,
		 char * export_filename,
		 int width, int height,
		 char * image_filename)
{
	char command[STRING_BLOCK],data_filename[STRING_BLOCK];
	char xlabelstr[STRING_BLOCK],ylabelstr[STRING_BLOCK];
	FILE * fp;
	int retval=0;
	char * plot_filename = "gnuplot.plot";

	/* remove the header */
	sprintf((char*)data_filename,"%s","gnuplot2.csv");
	trim_file_header(export_filename, data_filename);

	fp = fopen(plot_filename,"w");
	if (fp!=NULL) {

		/* create the gnuplot file */

		fprintf(fp,"%s","reset\n");
		fprintf(fp,"set title \"%s\"\n",title);

		if (subtitle != NULL) {
			fprintf(fp,"set label '%s' at screen 0.47, screen 0.90\n",
					subtitle);
		}

		sprintf(xlabelstr,"%s",	get_text_from_identifier(DATE));
		xlabelstr[0] = toupper(xlabelstr[0]);

		sprintf(ylabelstr,"%s",	get_text_from_identifier(BALANCE));
		ylabelstr[0] = toupper(ylabelstr[0]);

		fprintf(fp,"%s","set lmargin 11\n");
		fprintf(fp,"%s","set rmargin 2\n");
		fprintf(fp,"set xlabel \"%s\"\n",xlabelstr);
		fprintf(fp,"set ylabel \"%s\"\n",ylabelstr);
		fprintf(fp,"%s","set grid\n");
		fprintf(fp,"%s","set key off\n");
		fprintf(fp,"%s","set xdata time\n");    
		fprintf(fp,"%s","set timefmt \"%Y-%m\"\n");
		fprintf(fp,"%s","set format x \"%Y-%m\"\n");
		fprintf(fp,"set terminal png size %d,%d\n",width,height);
		fprintf(fp,"set output '%s'\n",image_filename);
		fprintf(fp,"plot '%s' using 1:%d with lines\n",
				data_filename,balance_field+1);

		fclose(fp);

		/* run gnuplot */

		sprintf((char*)command,"gnuplot %s",plot_filename);
		retval=system(command);
		plot_cleanup();
	}
	return retval;
}

int plot_distribution(char * title, char * subtitle,
					  int width, int height,
					  char * image_filename,
					  int * distribution, double maximum,
					  int distribution_size)
{
	char command[STRING_BLOCK],data_filename[STRING_BLOCK];
	char amountstr[STRING_BLOCK];
	FILE * fp;
	int i, retval=0;
	char * plot_filename = "gnuplot.plot";
	double increment = maximum / distribution_size;

	/* remove the header */
	sprintf((char*)data_filename,"%s","gnuplot2.csv");

	fp = fopen(data_filename,"w");
	if (fp != NULL) {
		for (i = 0; i < distribution_size; i++) {
			fprintf(fp,"%.6f %d\n",i*increment,distribution[i]);
		}
		fclose(fp);
	}

	fp = fopen(plot_filename,"w");
	if (fp!=NULL) {

		/* create the gnuplot file */

		fprintf(fp,"%s","reset\n");
		fprintf(fp,"set title \"%s\"\n",title);

		if (subtitle != NULL) {
			fprintf(fp,"set label '%s' at screen 0.47, screen 0.90\n",
					subtitle);
		}

		fprintf(fp,"%s","set lmargin 9\n");
		fprintf(fp,"%s","set rmargin 2\n");
		sprintf(amountstr,"%s",get_text_from_identifier(AMOUNT));
		if (amountstr!=0) {
			if (strlen(amountstr)>0) {
				amountstr[0] = toupper(amountstr[0]);
				fprintf(fp,"set xlabel \"%s\"\n", amountstr);
			}
		}
		fprintf(fp,"set ylabel \"%s\"\n",
				get_text_from_identifier(FREQUENCY));
		fprintf(fp,"%s","set grid\n");
		fprintf(fp,"%s","set key off\n");
		fprintf(fp,"set terminal png size %d,%d\n",width,height);
		fprintf(fp,"set output '%s'\n",image_filename);
		fprintf(fp,"plot '%s' using 1:2 with lines\n",
				data_filename);

		fclose(fp);

		/* run gnuplot */

		sprintf((char*)command,"gnuplot %s",plot_filename);
		retval=system(command);
		plot_cleanup();
	}
	return retval;
}

int plot_volume(char * title, char * subtitle,
				char * export_filename,
				int width, int height,
				char * image_filename)
{
	char command[STRING_BLOCK],data_filename[STRING_BLOCK];
	char xlabelstr[STRING_BLOCK],ylabelstr[STRING_BLOCK];
	FILE * fp;
	int retval=0;
	char * plot_filename = "gnuplot.plot";

	/* remove the header */
	sprintf((char*)data_filename,"%s","gnuplot2.csv");
	trim_file_header(export_filename, data_filename);

	fp = fopen(plot_filename,"w");
	if (fp!=NULL) {

		/* create the gnuplot file */

		fprintf(fp,"%s","reset\n");
		fprintf(fp,"set title \"%s\"\n",title);

		if (subtitle != NULL) {
			fprintf(fp,"set label '%s' at screen 0.47, screen 0.90\n",
					subtitle);
		}

		sprintf(xlabelstr,"%s",	get_text_from_identifier(DATE));
		xlabelstr[0] = toupper(xlabelstr[0]);

		sprintf(ylabelstr,"%s",	get_text_from_identifier(VOLUME));
		ylabelstr[0] = toupper(ylabelstr[0]);

		fprintf(fp,"%s","set lmargin 9\n");
		fprintf(fp,"%s","set rmargin 2\n");
		fprintf(fp,"set xlabel \"%s\"\n",xlabelstr);
		fprintf(fp,"set ylabel \"%s\"\n",ylabelstr);
		fprintf(fp,"%s","set grid\n");
		fprintf(fp,"%s","set key off\n");
		fprintf(fp,"%s","set xdata time\n");    
		fprintf(fp,"%s","set timefmt \"%Y-%m\"\n");
		fprintf(fp,"%s","set format x \"%Y-%m\"\n");
		fprintf(fp,"set terminal png size %d,%d\n",width,height);
		fprintf(fp,"set output '%s'\n",image_filename);
		fprintf(fp,"plot '%s' using 1:2 with lines\n",
				data_filename);

		fclose(fp);

		/* run gnuplot */

		sprintf((char*)command,"gnuplot %s",plot_filename);
		retval=system(command);
		plot_cleanup();
	}
	return retval;
}
